@@ -1,9 +1,9 @@
 // Libraries
 import { Observable, of, timer, merge, from } from 'rxjs';
 import { flatten, map as lodashMap, isArray, isString } from 'lodash';
-import { map, catchError, takeUntil, mapTo, share, finalize } from 'rxjs/operators';
+import { map, catchError, takeUntil, mapTo, share, finalize, tap } from 'rxjs/operators';
 // Utils & Services
-import { getBackendSrv } from 'app/core/services/backend_srv';
+import { backendSrv } from 'app/core/services/backend_srv';
 // Types
 import {
   DataSourceApi,
@@ -18,7 +18,10 @@ import {
   DataFrame,
   guessFieldTypes,
 } from '@grafana/data';
+import { toDataQueryError } from '@grafana/runtime';
+import { emitDataRequestEvent } from './analyticsProcessor';
 import { ExpressionDatasourceID, expressionDatasource } from 'app/features/expressions/ExpressionDatasource';
+import { ExpressionQuery } from 'app/features/expressions/types';
 
 type MapOfResponsePackets = { [str: string]: DataQueryResponse };
 
@@ -31,7 +34,7 @@ interface RunningQueryState {
  * This function should handle composing a PanelData from multiple responses
  */
 export function processResponsePacket(packet: DataQueryResponse, state: RunningQueryState): RunningQueryState {
-  const request = state.panelData.request;
+  const request = state.panelData.request!;
   const packets: MapOfResponsePackets = {
     ...state.packets,
   };
@@ -45,8 +48,8 @@ export function processResponsePacket(packet: DataQueryResponse, state: RunningQ
   const range = { ...request.range };
   const timeRange = isString(range.raw.from)
     ? {
-        from: dateMath.parse(range.raw.from, false),
-        to: dateMath.parse(range.raw.to, true),
+        from: dateMath.parse(range.raw.from, false)!,
+        to: dateMath.parse(range.raw.to, true)!,
         raw: range.raw,
       }
     : range;
@@ -78,7 +81,7 @@ export function processResponsePacket(packet: DataQueryResponse, state: RunningQ
  * It will
  *  * Merge multiple responses into a single DataFrame array based on the packet key
  *  * Will emit a loading state if no response after 50ms
- *  * Cancel any still runnning network requests on unsubscribe (using request.requestId)
+ *  * Cancel any still running network requests on unsubscribe (using request.requestId)
  */
 export function runRequest(datasource: DataSourceApi, request: DataQueryRequest): Observable<PanelData> {
   let state: RunningQueryState = {
@@ -112,13 +115,15 @@ export function runRequest(datasource: DataSourceApi, request: DataQueryRequest)
       return state.panelData;
     }),
     // handle errors
-    catchError(err =>
-      of({
+    catchError(err => {
+      console.error('runRequest.catchError', err);
+      return of({
         ...state.panelData,
         state: LoadingState.Error,
-        error: processQueryError(err),
-      })
-    ),
+        error: toDataQueryError(err),
+      });
+    }),
+    tap(emitDataRequestEvent(datasource)),
     // finalize is triggered when subscriber unsubscribes
     // This makes sure any still running network requests are cancelled
     finalize(cancelNetworkRequestsOnUnsubscribe(request)),
@@ -129,18 +134,12 @@ export function runRequest(datasource: DataSourceApi, request: DataQueryRequest)
   // If 50ms without a response emit a loading state
   // mapTo will translate the timer event into state.panelData (which has state set to loading)
   // takeUntil will cancel the timer emit when first response packet is received on the dataObservable
-  return merge(
-    timer(200).pipe(
-      mapTo(state.panelData),
-      takeUntil(dataObservable)
-    ),
-    dataObservable
-  );
+  return merge(timer(200).pipe(mapTo(state.panelData), takeUntil(dataObservable)), dataObservable);
 }
 
 function cancelNetworkRequestsOnUnsubscribe(req: DataQueryRequest) {
   return () => {
-    getBackendSrv().resolveCancelerIfExists(req.requestId);
+    backendSrv.resolveCancelerIfExists(req.requestId);
   };
 }
 
@@ -148,7 +147,7 @@ export function callQueryMethod(datasource: DataSourceApi, request: DataQueryReq
   // If any query has an expression, use the expression endpoint
   for (const target of request.targets) {
     if (target.datasource === ExpressionDatasourceID) {
-      return expressionDatasource.query(request);
+      return expressionDatasource.query(request as DataQueryRequest<ExpressionQuery>);
     }
   }
 
@@ -157,32 +156,8 @@ export function callQueryMethod(datasource: DataSourceApi, request: DataQueryReq
   return from(returnVal);
 }
 
-export function processQueryError(err: any): DataQueryError {
-  const error = (err || {}) as DataQueryError;
-
-  if (!error.message) {
-    if (typeof err === 'string' || err instanceof String) {
-      return { message: err } as DataQueryError;
-    }
-
-    let message = 'Query error';
-    if (error.message) {
-      message = error.message;
-    } else if (error.data && error.data.message) {
-      message = error.data.message;
-    } else if (error.data && error.data.error) {
-      message = error.data.error;
-    } else if (error.status) {
-      message = `Query error: ${error.status} ${error.statusText}`;
-    }
-    error.message = message;
-  }
-
-  return error;
-}
-
 /**
- * All panels will be passed tables that have our best guess at colum type set
+ * All panels will be passed tables that have our best guess at column type set
  *
  * This is also used by PanelChrome for snapshot support
  */
@@ -196,9 +171,11 @@ export function getProcessedDataFrames(results?: DataQueryResponseData[]): DataF
   for (const result of results) {
     const dataFrame = guessFieldTypes(toDataFrame(result));
 
-    // clear out any cached calcs
-    for (const field of dataFrame.fields) {
-      field.calcs = null;
+    if (dataFrame.fields && dataFrame.fields.length) {
+      // clear out the cached info
+      for (const field of dataFrame.fields) {
+        field.state = null;
+      }
     }
 
     dataFrames.push(dataFrame);
@@ -207,7 +184,7 @@ export function getProcessedDataFrames(results?: DataQueryResponseData[]): DataF
   return dataFrames;
 }
 
-export function preProcessPanelData(data: PanelData, lastResult: PanelData): PanelData {
+export function preProcessPanelData(data: PanelData, lastResult?: PanelData): PanelData {
   const { series } = data;
 
   //  for loading states with no data, use last result
@@ -216,12 +193,21 @@ export function preProcessPanelData(data: PanelData, lastResult: PanelData): Pan
       lastResult = data;
     }
 
-    return { ...lastResult, state: LoadingState.Loading };
+    return {
+      ...lastResult,
+      state: LoadingState.Loading,
+      request: data.request,
+    };
   }
 
   // Make sure the data frames are properly formatted
+  const STARTTIME = performance.now();
+  const processedDataFrames = getProcessedDataFrames(series);
+  const STOPTIME = performance.now();
+
   return {
     ...data,
-    series: getProcessedDataFrames(series),
+    series: processedDataFrames,
+    timings: { dataProcessingTime: STOPTIME - STARTTIME },
   };
 }
